#include <stdio.h>
/*
  Modify the sort program to handle a -r flag, which indicates sorting in reverse (decreasing) order. Be sure that -r works with -n.

    Add the option -f to fold upper and lower case together, so that case distinctions are not made during sorting; for example, a and A compare equal.

      Add the -d ("directory order") option, which makes comparisons only on letters, numbers and blanks. Make sure it works in conjunction with -f .
 */


#define MAXLINES 1000
char *lineptr[MAXLINES];

int readlines(char *lineptr[], int nlines);
void writelines(char *lineptr[], int nlines);

void qsort(void *lineptr[], int left, int right,
           int (*comp)(void *, void *));

int stringcmp(char *, char *);
int stringcmpi(char *, char *);
int isdir(int c);
int dircmp(char *, char *);
int numcmp(char *, char *);

int reverse = 0;
int fold = 0;

int main(int argc, char *argv[])
{
  int nlines;
  int numeric = 0;
  int directory = 0;
  int (*comp)(void *, void *);
  

  while(--argc > 0){
    if((*++argv)[0] == '-'){
      while(*++(argv[0]) != '\0'){
        switch(**argv){
        case 'd':
          directory = 1;
          break;
        case 'f':
          fold = 1;
          break;
        case 'n':
          numeric = 1;
          break;
        case 'r':
          reverse = 1;
          break;
        default:
          fprintf(sderr, "error: unknow option -%c.\n", **argv);
          break;
        }
      }
    }
  }

  if(numeric)
    comp = (int (*)(void *, void *)) numcmp;
  else if(directory)
    comp = (int (*)(void *, void *)) dircmp;
  else
    comp = (int (*)(void *, void *)) stringcmp;

  if(nlines = readlines(lineptr, MAXLINES) >= 0){
    qsort((void **)lineptr, 0, nlines - 1, comp);
    writelines(lineptr, nlines);
  }
  else{
    fprintf(stderr, "input too big to sort.\n");
  }
  
  return 0;
}


#define MAXLEN 1000 /* max length of input line */

#define ALLOCSIZE 10000 /* size of available space */

static char allocbuf[ALLOCSIZE];  /* storage for alloc */
static char *allocp = allocbuf;  /* next free position */

char *alloc(int n)
{
  if(allocbuf + ALLOCSIZE - allocp >= n){
    allocp += n;
    return allocp -n;
  }
  else
    return 0;
}

int getline(char s[], int lim)
{
  char c;
  int i;
  for(i = 0; i < lim - 1 && (c = getchar()) != EOF &&  c != '\n'; ++i){
    s[i] = c;
  }

  if(c == '\n'){
    s[i] = c;
    ++i;
  }
  s[i] = '\0';

  while(c != EOF && c != '\n')
    c = getchar();
  return i;
}
    


int readlines(char *lineptr[], int maxlines)
{
  int line[MAXLEN];
  int len, nlines;
  char *p;
  
  nlines = 0;
  while((len = getline(line, MAXLEN)) > 0){
    if(nlines >= maxlines || (p = alloc(len)) == NULL)
      return -1;
    else{
      line[len - 1] = '\0';  /* delete newline */
      strcpy(p, line);
      lineptr[nlines++] = p;
    }
  }
  return nlines;
  
}


void writelines(char *lineptr[], int nlines)
{
  int i;
  for(i = 0; i < nlines; ++i)
    printf("%s\n". lineptr[i]);
}

void swap(void *v[], int i, int j)
{
  void *temp;
  temp = v[i];
  v[i] = v[j];
  v[j] = temp;
}


void qsort(void *v[], int left, int right,
           int (*comp)(void *, void *))
{
  int i, last;

  if(left >= right)
    return;
  swap(left, (left + right) / 2);

  last = left;
  for(i = left + 1; i <= right; ++i){
    if((*comp)(v[i], v[left]) < 0)
      swap(v, ++last, i);
  }

  swap(v, left, last);
  qsort(v, left, last - 1, comp);
  qsort(v, last + 1, right, comp);
}

int numcmp(char *s1, char *s2)
{
  double v1, v2;

  v1 = atof(s1);
  v2 = atof(s2);

  if(v1 < v2)
    return (reverse)? 1 : -1;
  else if(v1 > v2)
    return (reverse)? -1 : 1;
  else
    return 0;
}

int isdir(int c)
{
  return isalpha(c) || isdigit(c) || isspace(c);
}

int dircmp(char *s1, char *s2);
{
  int d;

  do{
    while(!isdir(*s1) && s1 != '\0')
      ++s1;
    while(!isdir(*s2) && s2 != '\0')
      ++s2;
    if(fold)
      d = toupper(*s1) - toupper(*s2);
    else
      d = *s1 - *s2;
    ++s1;
    ++s2;
  }while(d == 0 && *s1 != '\0' && *s2 != '\0');

  return d;
}
  

int stringcmp(char *s1, char *s2)
{
  if(fold)
    return stringcmpi(s1, s2);

  d= strcmp(s1, s2);

  return (reverse)? -d : d;
}

int stringcmpi(char *s1, char *s2)
{
  	int d;

	while (toupper(*s1) == toupper(*s2)) {
		if (*s1 == '\0')
			return 0;
		s1++;
		s2++;
	}
	d = toupper(*s1) - toupper(*s2);

	return (reverse) ? -d : d;
}

